#include "RF1A.h"
#include <msp430.h>

// Data from SmartRF Studip 7
// Modulated = true
// Channel number = 0
// PA ramping = false
// Modulation format = 2-GFSK
// Packet length mode = Variable packet length mode. Packet length configured by the first byte after sync word
// Address config = No address check
// Base frequency = 431.425201
// Device address = 0
// Carrier frequency = 431.425201
// CRC enable = true
// Packet length = 255
// Data format = Normal mode
// RX filter BW = 232.142857
// Whitening = true
// Data rate = 99.9756
// Preamble count = 8
// CRC autoflush = false
// TX power = 10
// Channel spacing = 399.902344
// Manchester enable = false
// Sync word qualifier mode = 30/32 sync word bits detected
// Sync word value = 0x1234
// Deviation = 47.607422
// Rf settings for CC430
RF_SETTINGS rfSettings = {
    0x2E,  // IOCFG2        GDO2 Output Configuration
    0x2E,  // IOCFG1        GDO1 Output Configuration
    0x2E,  // IOCFG0        GDO0 Output Configuration
    0x47,  // FIFOTHR       RX FIFO and TX FIFO Thresholds
    0x34,  // SYNC1         Sync Word, High Byte
    0x12,  // SYNC0         Sync Word, Low Byte
    0xFF,  // PKTLEN        Packet Length
    0x04,  // PKTCTRL1      Packet Automation Control
    0x45,  // PKTCTRL0      Packet Automation Control
    0x00,  // ADDR          Device Address
    0x00,  // CHANNR        Channel Number
    0x08,  // FSCTRL1       Frequency Synthesizer Control
    0x00,  // FSCTRL0       Frequency Synthesizer Control
    0x10,  // FREQ2         Frequency Control Word, High Byte
    0x94,  // FREQ1         Frequency Control Word, Middle Byte
    0xE1,  // FREQ0         Frequency Control Word, Low Byte
    0x7B,  // MDMCFG4       Modem Configuration
    0xF8,  // MDMCFG3       Modem Configuration
    0x13,  // MDMCFG2       Modem Configuration
    0x43,  // MDMCFG1       Modem Configuration
    0xF8,  // MDMCFG0       Modem Configuration
    0x47,  // DEVIATN       Modem Deviation Setting
    0x07,  // MCSM2         Main Radio Control State Machine Configuration
    0x00,  // MCSM1         Main Radio Control State Machine Configuration
    0x18,  // MCSM0         Main Radio Control State Machine Configuration
    0x1D,  // FOCCFG        Frequency Offset Compensation Configuration
    0x1C,  // BSCFG         Bit Synchronization Configuration
    0xC7,  // AGCCTRL2      AGC Control
    0x00,  // AGCCTRL1      AGC Control
    0xB2,  // AGCCTRL0      AGC Control
    0x87,  // WOREVT1       High Byte Event0 Timeout
    0x6B,  // WOREVT0       Low Byte Event0 Timeout
    0xFB,  // WORCTRL       Wake On Radio Control
    0xB6,  // FREND1        Front End RX Configuration
    0x10,  // FREND0        Front End TX Configuration
    0xEA,  // FSCAL3        Frequency Synthesizer Calibration
    0x2A,  // FSCAL2        Frequency Synthesizer Calibration
    0x00,  // FSCAL1        Frequency Synthesizer Calibration
    0x1F,  // FSCAL0        Frequency Synthesizer Calibration
    0x59,  // FSTEST
    0x7F,  // PTEST
    0x3F,  // AGCTEST
    0x81,  // TEST2
    0x35,  // TEST1
    0x09,  // TEST0
};



// *****************************************************************************
// @fn          Strobe
// @brief       Send a command strobe to the radio.
// @param       unsigned char strobe        The strobe command to be sent
// @return      unsigned char statusByte    The status byte that follows the strobe
// *****************************************************************************
unsigned char Strobe(unsigned char strobe)
{
  unsigned char statusByte = 0;
  unsigned int  gdo_state;
  
  // Check for valid strobe command 
  if((strobe == 0xBD) || ((strobe >= RF_SRES) && (strobe <= RF_SNOP)))
  {
    // Clear the Status read flag 
    RF1AIFCTL1 &= ~(RFSTATIFG);    
    
    // Wait for radio to be ready for next instruction
    while( !(RF1AIFCTL1 & RFINSTRIFG));
    
    // Write the strobe instruction
    if ((strobe > RF_SRES) && (strobe < RF_SNOP))
    {
      gdo_state = ReadSingleReg(IOCFG2);    // buffer IOCFG2 state
      WriteSingleReg(IOCFG2, 0x29);         // chip-ready to GDO2
      
      RF1AINSTRB = strobe; 
      if ( (RF1AIN&0x04)== 0x04 )           // chip at sleep mode
      {
        if ( (strobe == RF_SXOFF) || (strobe == RF_SPWD) || (strobe == RF_SWOR) ) { }
        else  	
        {
          while ((RF1AIN&0x04)== 0x04);     // chip-ready ?
          // Delay for ~810usec at 1.05MHz CPU clock, see Section 22.3.3.7 
		  // Radio Control – Crystal Control of the CC430 User’s Guide (SLAU 259)”
          __delay_cycles(850);	            
        }
      }
      WriteSingleReg(IOCFG2, gdo_state);    // restore IOCFG2 setting
    
      while( !(RF1AIFCTL1 & RFSTATIFG) );
    }
    else		                    // chip active mode (SRES)
    {	
      RF1AINSTRB = strobe; 	   
    }
    statusByte = RF1ASTATB;
  }
  return statusByte;
}

// *****************************************************************************
// @fn          ReadSingleReg
// @brief       Read a single byte from the radio register
// @param       unsigned char addr      Target radio register address
// @return      unsigned char data_out  Value of byte that was read
// *****************************************************************************
unsigned char ReadSingleReg(unsigned char addr)
{
  unsigned char data_out;
  
  // Check for valid configuration register address, 0x3E refers to PATABLE 
  if ((addr <= 0x2E) || (addr == 0x3E))
    // Send address + Instruction + 1 dummy byte (auto-read)
    RF1AINSTR1B = (addr | RF_SNGLREGRD);    
  else
    // Send address + Instruction + 1 dummy byte (auto-read)
    RF1AINSTR1B = (addr | RF_STATREGRD);    
  
  while (!(RF1AIFCTL1 & RFDOUTIFG) );
  data_out = RF1ADOUTB;                    // Read data and clears the RFDOUTIFG

  return data_out;
}

// *****************************************************************************
// @fn          WriteSingleReg
// @brief       Write a single byte to a radio register
// @param       unsigned char addr      Target radio register address
// @param       unsigned char value     Value to be written
// @return      none
// *****************************************************************************
void WriteSingleReg(unsigned char addr, unsigned char value)
{   
  while (!(RF1AIFCTL1 & RFINSTRIFG));       // Wait for the Radio to be ready for next instruction
  RF1AINSTRB = (addr | RF_SNGLREGWR);	    // Send address + Instruction

  RF1ADINB = value; 			    // Write data in 

  __no_operation(); 
}
        
// *****************************************************************************
// @fn          ReadBurstReg
// @brief       Read multiple bytes to the radio registers
// @param       unsigned char addr      Beginning address of burst read
// @param       unsigned char *buffer   Pointer to data table
// @param       unsigned char count     Number of bytes to be read
// @return      none
// *****************************************************************************
void ReadBurstReg(unsigned char addr, unsigned char *buffer, unsigned char count)
{
  unsigned int i;
  if(count > 0)
  {
    while (!(RF1AIFCTL1 & RFINSTRIFG));       // Wait for INSTRIFG
    RF1AINSTR1B = (addr | RF_REGRD);          // Send addr of first conf. reg. to be read 
                                              // ... and the burst-register read instruction
    for (i = 0; i < (count-1); i++)
    {
      while (!(RFDOUTIFG&RF1AIFCTL1));        // Wait for the Radio Core to update the RF1ADOUTB reg
      buffer[i] = RF1ADOUT1B;                 // Read DOUT from Radio Core + clears RFDOUTIFG
                                              // Also initiates auo-read for next DOUT byte
    }
    buffer[count-1] = RF1ADOUT0B;             // Store the last DOUT from Radio Core  
  }
}  

// *****************************************************************************
// @fn          WriteBurstReg
// @brief       Write multiple bytes to the radio registers
// @param       unsigned char addr      Beginning address of burst write
// @param       unsigned char *buffer   Pointer to data table
// @param       unsigned char count     Number of bytes to be written
// @return      none
// *****************************************************************************
void WriteBurstReg(unsigned char addr, unsigned char *buffer, unsigned char count)
{  
  unsigned char i;

  if(count > 0)
  {
    while (!(RF1AIFCTL1 & RFINSTRIFG));       // Wait for the Radio to be ready for next instruction
//    RF1AINSTRW = ((addr | RF_REGWR)<<8 ) + buffer[0]; // Send address + Instruction
    RF1AINSTRW = ((addr | RF_REGWR)<<8 ) + count; // Send address + Instruction
  
    for (i = 0; i < count; i++)
    {
      RF1ADINB = buffer[i];                   // Send data
      while (!(RFDINIFG & RF1AIFCTL1));       // Wait for TX to finish
    } 
    i = RF1ADOUTB;                            // Reset RFDOUTIFG flag which contains status byte  
  }
}

// *****************************************************************************
// @fn          ResetRadioCore
// @brief       Reset the radio core using RF_SRES command
// @param       none
// @return      none
// *****************************************************************************
void ResetRadioCore (void)
{
  Strobe(RF_SRES);                          // Reset the Radio Core
  Strobe(RF_SNOP);                          // Reset Radio Pointer
}

// *****************************************************************************
// @fn          WriteRfSettings
// @brief       Write the minimum set of RF configuration register settings
// @param       RF_SETTINGS *pRfSettings  Pointer to the structure that holds the rf settings
// @return      none
// *****************************************************************************

void WriteRfSettings(RF_SETTINGS *pRfSettings) {
	WriteSingleReg(IOCFG2, pRfSettings->iocfg2);  //GDO2 Output Configuration
	WriteSingleReg(IOCFG1, pRfSettings->iocfg1);  //GDO1 Output Configuration
	WriteSingleReg(IOCFG0, pRfSettings->iocfg0);  //GDO0 Output Configuration
	WriteSingleReg(FIFOTHR, pRfSettings->fifothr); //RX FIFO and TX FIFO Thresholds
	WriteSingleReg(SYNC1, pRfSettings->sync1);   //Sync Word, High Byte
	WriteSingleReg(SYNC0, pRfSettings->sync0);   //Sync Word, Low Byte
	WriteSingleReg(PKTLEN, pRfSettings->pktlen);  //Packet Length
	WriteSingleReg(PKTCTRL1, pRfSettings->pktctrl1);//Packet Automation Control
	WriteSingleReg(PKTCTRL0, pRfSettings->pktctrl0);//Packet Automation Control
	WriteSingleReg(ADDR, pRfSettings->addr);    //Device Address
	WriteSingleReg(CHANNR, pRfSettings->channr);  //Channel Number
	WriteSingleReg(FSCTRL1, pRfSettings->fsctrl1); //Frequency Synthesizer Control
	WriteSingleReg(FSCTRL0, pRfSettings->fsctrl0); //Frequency Synthesizer Control
	WriteSingleReg(FREQ2, pRfSettings->freq2);   //Frequency Control Word, High Byte
	WriteSingleReg(FREQ1, pRfSettings->freq1);   //Frequency Control Word, Middle Byte
	WriteSingleReg(FREQ0, pRfSettings->freq0);   //Frequency Control Word, Low Byte
	WriteSingleReg(MDMCFG4, pRfSettings->mdmcfg4); //Modem Configuration
	WriteSingleReg(MDMCFG3, pRfSettings->mdmcfg3); //Modem Configuration
	WriteSingleReg(MDMCFG2, pRfSettings->mdmcfg2); //Modem Configuration
	WriteSingleReg(MDMCFG1, pRfSettings->mdmcfg1); //Modem Configuration
	WriteSingleReg(MDMCFG0, pRfSettings->mdmcfg0); //Modem Configuration
	WriteSingleReg(DEVIATN, pRfSettings->deviatn); //Modem Deviation Setting
	WriteSingleReg(MCSM2, pRfSettings->mcsm2);   //Main Radio Control State Machine Configuration
	WriteSingleReg(MCSM1, pRfSettings->mcsm1);   //Main Radio Control State Machine Configuration
	WriteSingleReg(MCSM0, pRfSettings->mcsm0);   //Main Radio Control State Machine Configuration
	WriteSingleReg(FOCCFG, pRfSettings->foccfg);  //Frequency Offset Compensation Configuration
	WriteSingleReg(BSCFG, pRfSettings->bscfg);   //Bit Synchronization Configuration
	WriteSingleReg(AGCCTRL2, pRfSettings->agcctrl2);//AGC Control
	WriteSingleReg(AGCCTRL1, pRfSettings->agcctrl1);//AGC Control
	WriteSingleReg(AGCCTRL0, pRfSettings->agcctrl0);//AGC Control
	WriteSingleReg(WOREVT1, pRfSettings->worevt1); //High Byte Event0 Timeout
	WriteSingleReg(WOREVT0, pRfSettings->worevt0); //Low Byte Event0 Timeout
	WriteSingleReg(WORCTRL, pRfSettings->worctrl); //Wake On Radio Control
	WriteSingleReg(FREND1, pRfSettings->frend1);  //Front End RX Configuration
	WriteSingleReg(FREND0, pRfSettings->frend0);  //Front End TX Configuration
	WriteSingleReg(FSCAL3, pRfSettings->fscal3);  //Frequency Synthesizer Calibration
	WriteSingleReg(FSCAL2, pRfSettings->fscal2);  //Frequency Synthesizer Calibration
	WriteSingleReg(FSCAL1, pRfSettings->fscal1);  //Frequency Synthesizer Calibration
	WriteSingleReg(FSCAL0, pRfSettings->fscal0);  //Frequency Synthesizer Calibration
	WriteSingleReg(FSTEST, pRfSettings->fstest);
	WriteSingleReg(PTEST, pRfSettings->ptest);
	WriteSingleReg(AGCTEST, pRfSettings->agctest);
	WriteSingleReg(TEST2, pRfSettings->test2);
	WriteSingleReg(TEST1, pRfSettings->test1);
	WriteSingleReg(TEST0, pRfSettings->test0);
}

// *****************************************************************************
// @fn          WritePATable
// @brief       Write data to power table
// @param       unsigned char value		Value to write
// @return      none
// *****************************************************************************
void WriteSinglePATable(unsigned char value)
{
  while( !(RF1AIFCTL1 & RFINSTRIFG));
  RF1AINSTRW = 0x3E00 + value;              // PA Table single write
  
  while( !(RF1AIFCTL1 & RFINSTRIFG));
  RF1AINSTRB = RF_SNOP;                     // reset PA_Table pointer
}

// *****************************************************************************
// @fn          WritePATable
// @brief       Write to multiple locations in power table 
// @param       unsigned char *buffer	Pointer to the table of values to be written 
// @param       unsigned char count	Number of values to be written
// @return      none
// *****************************************************************************
void WriteBurstPATable(unsigned char *buffer, unsigned char count)
{
  volatile char i = 0; 
  
  while( !(RF1AIFCTL1 & RFINSTRIFG));
  RF1AINSTRW = 0x7E00 + buffer[i];          // PA Table burst write   

  for (i = 1; i < count; i++)
  {
    RF1ADINB = buffer[i];                   // Send data
    while (!(RFDINIFG & RF1AIFCTL1));       // Wait for TX to finish
  } 
  i = RF1ADOUTB;                            // Reset RFDOUTIFG flag which contains status byte

  while( !(RF1AIFCTL1 & RFINSTRIFG));
  RF1AINSTRB = RF_SNOP;                     // reset PA Table pointer
}


void InitRadio(void) {
	  WriteRfSettings(&rfSettings);		//стравим радио настройки
	  WriteSinglePATable(PATABLE_VAL);				//еще скинем туда PATABLE
}

void TransmitOn(void) {
	RF1AIES |= BIT9;                          	//спад RFIFG9
    RF1AIFG &= ~BIT9;                         	//сброс бита окончания обработки прерывания
    RF1AIE &= ~BIT9;                           	//разрешаем прерывание по отправке
    Strobe( RF_STX );                         	//переключаемся в передачу
    __no_operation();
	while(!(RF1AIFG & BIT9)) {__no_operation();} //дожидаемся окончания передачи пакета
	ReceiveOn();								//вернемся в режим приема
}

void ReceiveOn(void) {
	RF1AIES |= BIT9;                          	//спад RFIFG9
    RF1AIFG &= ~BIT9;                         	//сброс бита окончания обработки прерывания
    RF1AIE  |= BIT9;                          	//разрешаем прерывания
    Strobe( RF_SRX );							//переключаем сс1101 в прием
    __no_operation();
}

void ReceiveOff(void) {
	RF1AIE &= ~BIT9;                          	//Запрет прерывания по приему
	RF1AIFG &= ~BIT9;                         	//очистим бит обработки прерывания
	RF1AIES &= ~BIT9;
	Strobe(RF_SIDLE);							//чутка пусть передохнет
	Strobe(RF_SFRX);							//очистим буфер FIFO RX
}
